#
# QTFairUse6 1.0 2006-08-28
# Copyright (C) 2006 Igor Skochinsky <skochinsky@mail.ru>
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from pydbg import *
from pydbg.defines import *

import sys, struct

error = lambda msg: sys.stderr.write("ERROR> " + msg + "\n") or sys.exit(1)
outf = None
fileno = 0
offset = 0
offsets1 = [0x4FB39A, 0x4FCB7A] #debug check
offsets2 = [0x976930, 0x989760] #frame copy
offsets3 = [0x4b0ad0, 0x4B1180] #end-of-track function
versions = ["6.0.4", "6.0.5"]

def find_pid(dbg, name):
    namel = name.lower()
    found_target = False
    for (pid, proc_name) in dbg.enumerate_processes():
        if proc_name.lower() == namel:
            found_target = True
            break
    if found_target:
        return pid
    else:
        return -1

def getDwordAt(dbg, addr):
    return struct.unpack("<I", dbg.read_process_memory(addr, 4))[0]

def changeTrack():
    global fileno, outf, offset, outfname
    if outf:
        outf.close()
        outf = None
    fileno+=1
    outfname = "dump_%02d.aac"%fileno
    offset = 0

def DoAACFrame (dbg):
    global fileno, outf, offset, outfname
    # unknown, bufptr, size
    size = dbg.get_arg(2)
    ptr = getDwordAt(dbg, dbg.get_arg(1))
    if size>0 and ptr!=0:
        if fileno==0:
            changeTrack()
        if outf==None:
            print "New track: writing to %s"%outfname
            outf = file(outfname,"wb")
        #print "frame (%d bytes @ %08x)" % (size,offset)
        outf.write(dbg.read_process_memory(ptr,size))
        offset += size
    return DBG_CONTINUE

def EndOfTrack (dbg):
    print "End of track"
    changeTrack()
    return DBG_CONTINUE

def bp_check (dbg):
    print "Hit %08X" % dbg.context.Eip
    return DBG_CONTINUE

##############
#main code
##############

print "QTFairUse6 1.0 2006-08-28\nCopyright 2006 Igor Skochinsky <skochinsky@mail.ru>\n"

dbg = pydbg()

procname = "iTunes.exe"
pid = find_pid(dbg, procname)

if pid!=-1:
    print "attaching to %s(%d)" % (procname, pid)
    dbg.attach(pid)
    #dbg.debug_event_loop()
else:
    error("process '%s' not found." % procname)

ver = 0
found = False
for ver in xrange(len(offsets1)):
    b1 = dbg.read_process_memory(offsets1[ver], 1)
    if b1=='\x76': #jbe
        found = True
        break
    elif b1=='\xEB':
        patched = True
        break

if not found:
    if patched:
        #print "Already patched"
        pass
    else:
        error("Only iTunes %s to %s are supported"%(versions[0],versions[-1]))

print "Detected iTunes %s" % versions[ver]
dbg.write_process_memory(offsets1[ver], '\xEB')
dbg.bp_set(offsets2[ver], handler=DoAACFrame)
dbg.bp_set(offsets3[ver], handler=EndOfTrack)

print "Starting debugging"
dbg.run()

changeTrack()
print "Exiting"
